#!/bin/sh
# $Id: t-mailext-0.sh,v 1.17 2011/02/01 06:02:45 ca Exp $
#
# Copyright (c) 2010 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test various MAIL extensions
# ----------------------------------------
#
test -s stop && exit 1
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps COMMERCIAL
chkcto smtps SMI_PRI
chkcto smtps SMI_MAILEXT

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# clean up before running tests
cd ../qmgr || exit 1
rm -f ${MTMAP} ${QMGRCONFMAP} ${ACCMAP} ${ALIMAP}

# create aliases
${CAT} > ${ALI} <<EOF
postmaster:	local:
user:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	[127.0.0.1]
other.dom	[127.1.2.3]
EOF

chkiid()
{
  IID=$1

  for L in ${SSL} ${QML} ${SCL}
  do
    if grep " iid=${IID}," $L >/dev/null
    then
      :
    else
      $VERBOSE && echo "$0: ERROR: ${TEST}: missing iid=${IID} in $L"
      ERR=true
    fi
  done
}

chkiqdb()
{
  PRI=$1

  if grep "^max *\[ *$PRI\]= *[^0]$" ${QML} >/dev/null
  then
    :
  else
    $VERBOSE && echo "$0: ERROR: ${TEST}: missing iid=${IID} in $L"
    ERR=true
  fi
}


# ----------------------------------------
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SD}/smtp1.sh -Ssender@other.dom -Ruser@local.dom -Z smtpc2 -c '-M IID=InJectiOn1'
then
  chkiid InJectiOn1
else
  ERR=true
fi

if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SD}/smtp1.sh -1 -Ssender@other.dom -Ruser@local.dom -Z smtpc2 -c '-M PRI=12'
then
  chkiqdb 12
  if grep ' pri=12,' ${SSL} >/dev/null
  then
    :
  else
    $VERBOSE echo "$0: ERROR: ${TEST}: missing pri in $SSL"
    ERR=true
  fi
else
  ERR=true
fi

if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SD}/smtp1.sh -Ssender@other.dom -Ruser@local.dom -Z smtpc2 -c '-M RMIN=10_RMAX=120_QRET=360'
then
  :
else
  ERR=true
fi

if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
TEST=4
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SD}/smtp1.sh -1 -Ssender@other.dom -Ruser@local.dom -Z smtpc2 -c '-M RMIN=10_RMAX=120_QRET=360_IID=SomeID_PRI=1'
then
  chkiid SomeID
  chkiqdb 1
  if grep ' pri=1,' ${SSL} >/dev/null
  then
    :
  else
    $VERBOSE echo "$0: ERROR: ${TEST}: missing pri in $SSL"
    ERR=true
  fi
else
  ERR=true
fi

if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# this should not be accepted
TEST=5
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SD}/smtp1.sh -Ssender@other.dom -Ruser@local.dom -Z smtpc2 -c '-M RMIN=-1_RMAX=120_QRET=360' -A 0 -X 0 -E 0
then
  :
else
  ERR=true
fi

if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# this should not be accepted
TEST=6
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SD}/smtp1.sh -Ssender@other.dom -Ruser@local.dom -Z smtpc2 -c '-M IID=1_2' -A 0 -X 0 -E 0
then
  :
else
  ERR=true
fi

if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
