#!/bin/sh
# $Id: t-pm-msgreplace-1.sh,v 1.9 2011/10/20 15:07:14 ca Exp $
# Copyright (c) 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test message replacement (created by pmilter)
# uses smtpcnf.sh.
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps MTA_USE_PMILTER '(--enable-pmilter)'

OUT="t-pm-msgreplace-0.out"
MSG="t-pm-msgreplace-0.msg"
OK="t-pm-msgreplace-0.ok"
PMPRG="-p ../libpmilter/t-pmilter-1"
PMOPT=""

if test X"`uname`" != "XOSF1"
then
while getopts O:p:s:V FLAG
do
  case "${FLAG}" in
    O) PMOPT="${PMOPT} ${OPTARG}";;
    p) PMPRG="-p ${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || myexit 1
rm -f ${MTMAP}

if [ X"${MTA_CDB_GID}" != "X" ]
then
   CDB_GID="cdb_gid=${MTA_CDB_GID};"
else
   CDB_GID=""
fi

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }

smtps {
Log_Level = 19;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
policy_milter { socket {type=unix; path="pmilter.sock";} timeout=2; }
${CDB_GID}
}
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

# ----------------------------------------
# replace message
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${MSG} <<EOF
From: replace@from
To: replaced@to
Subject: fooled ya

body replacements
are mean
.
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -3 lmtpsock ${PMPRG} ${PMOPT} -P "-B ${MSG}" -Z 'smtpc2' -c '-E' -E 0 -A 1 -X 0 "$@" -S"a@b.c" -R"user@local.dom"
then
  ERR=false
  if test -s ${B1}
  then
    ${GREP} -v '^Return-Path: ' ${B1} > ${OUT}
    if ${DIFF} ${MSG} ${OUT} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: messages are different"
      ERR=true
    fi
  else
     ERR=true
  fi

else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "$0: test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# replace message, missing .\r\n
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${MSG} <<EOF
From: replace@from
To: replaced@to
Subject: fooled ya

body replacements
are mean
EOF
${CAT} ${MSG} > ${OK}
${CAT} >> ${OK} <<EOF
.
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -3 lmtpsock ${PMPRG} ${PMOPT} -P "-B ${MSG}" -Z 'smtpc2' -c '-E' -E 0 -A 1 -X 0 "$@" -S"a@b.c" -R"user@local.dom"
then
  ERR=false
  if test -s ${B1}
  then
    ${GREP} -v '^Return-Path: ' ${B1} > ${OUT}
    if ${DIFF} ${OK} ${OUT} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: messages are different"
      ERR=true
    fi
  else
     ERR=true
  fi

else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "$0: test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# replacement message is invalid
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${MSG} <<EOF
From: replace@from
To: replaced@to
Subject: fooled ya

body replacements
are mean and this is broken
EOF
${CAT} > ${OK} <<EOF
Return-Path: <a@b.c>
From: me
To: you
Subject: test

body
.
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -3 lmtpsock ${PMPRG} ${PMOPT} -P "-B ${MSG}" -Z 'smtpc2' -c '-E' -E 0 -A 1 -X 0 "$@" -S"a@b.c" -R"user@local.dom" -8 s
then
  ERR=false
  if test -s ${B1}
  then
     ${EGREP} -v '^Date: ' ${B1} |\
	 ${SED} -e '/^Received:/,/^	id /d' > ${OUT}
    if ${DIFF} ${OK} ${OUT} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: messages are different"
      ERR=true
    fi
  else
     ERR=true
  fi

else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "$0: test ${TEST} succeeded"
fi
fi



# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${OUT} ${MSG} ${OK}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  myexit 1
fi
