/*
 * $Id: sm-conf-expr.h,v 1.9 2013/10/23 22:13:38 ca Exp $
 */

#ifndef SM_CONF_EXPR_H
#define SM_CONF_EXPR_H 1

#define op_T	int

struct id_S {
	const char	*id_str;
	uint		 id_len;
	int			 id_idx;
	val_type_T	 id_vt;
};
typedef struct id_S id_T, *id_P;

union cval_U {
	op_T		 cv_op;
	uint32_t	 cv_int;
	bool		 cv_bool;
	id_T		 cv_id;
	const char	*cv_str;
};
typedef union cval_U cval_T, *cval_P;

/* access a cond.expr node in an expression */
#define CXN(expr, i)	((expr)->cex_nodes[i])

#define CEN_NARGS_MAX	3

/* (cond.expr) node */
/*
need a function signature, e.g.,
int=OP_ADD(int, int), bool=OP_AND(bool, bool)
hmm, is this already checked by the grammar?
*/
typedef struct cexn_S cexn_T, *cexn_P;
struct cexn_S {
	int			 cen_nargs;
	val_type_T	 cen_vt;
	cval_T		 cen_val;
	uint		 cen_args[CEN_NARGS_MAX];

	/* evaluated: needs less types (int, bool, str) */
	val_type_T	 cen_evt;
	cval_T		 cen_eval;
};

/* cond.expr */
typedef struct cexpr_S cexpr_T, *cexpr_P;
struct cexpr_S {
	uint	cex_total;
	uint	cex_next;
	uint	cex_rooti;
	cexn_P	cex_nodes;
};

/* for simpler id table specification */
#define ID_STR(x)	x, sizeof(x) - 1
/* as long as ide_val is used... */
#define ID_VAL(x)	x, 

#define IDEF_NONE	0x0000
#define IDEF_CONST_INT	0x0001
#define IDEF_CONST_BOOL	0x0002

struct sm_id_entry_S
{
	const char	*ide_id;
	uint		 ide_len;

	val_type_T	 ide_vt;

	/* hack, only ok for IDEF_CONST_{BOOL,INT} */
	int			 ide_val;

	/* pointer to "live" data */
	/*
	 *  How to we "link" variable names (identifiers) in an expression
	 *  to variables in the binary?
	 *  those variables could be
	 *  - global: easy: just add a pointer
	 *  - local: usually relative to a struct
	 *	  so we need a pointer to the struct (how if there are many instances?)
	 *    and an offset in the struct (simular to conf data, but that has only
	 *    one instance).
	 *    Do we need some "interpreter", e.g.,
	 *    if (struct_name == "rcpt_addr") base_ptr = rcpt_addr; else
	 *    if (struct_name == "qss_ta") base_ptr = qss_ta;
	 *    that "interpreter" needs to be invoked with the right "context"...
	 */

	/* for now: just an int value that is used by an "interpreter" */
	uint		 ide_idv;

	uint		 ide_flags;
};
typedef struct sm_id_entry_S	sm_id_entry_T, *sm_id_entry_P;

struct sm_eval_ctx_S
{
	sm_id_entry_P		 ec_idtab;
	size_t				 ec_idtabsz;
	sm_conf_idgetval_T	*ec_idgetval;
	void				*ec_idgetvalctx;
};
typedef struct sm_eval_ctx_S	sm_eval_ctx_T, *sm_eval_ctx_P;

sm_ret_T eval_exp(cexpr_P _expr, uint _exni, sm_eval_ctx_P _evalctx);

#endif /* SM_CONF_EXPR_H */
