/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-4.c,v 1.3 2014/04/19 15:50:42 ca Exp $")

#if SM_LIBCONF_ALONE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sm-conf.h"
#else /* SM_LIBCONF_ALONE */
#include "sm/string.h"
#include "sm/net.h"
#include "sm/sm-conf.h"
#include <stdio.h>
#endif /* SM_LIBCONF_ALONE */

/*
**  Just a test program to play around with libconf.
*/
enum vehicle_type_E
{
	AIRPLANE       = 1,
	BOAT           = 2,
	CAR            = 3
};

typedef enum vehicle_type_E vehicle_type_T;
typedef union vehicle_U
{
	vehicle_type_T vehicle_type;

	struct {
	       vehicle_type_T   air_type;
	       char            *air_flight;
	       int              air_passengers;
	} airplane;

	struct {
	       vehicle_type_T   boat_type;
	       int              boat_knots;
	       char            *boat_name;
	} boat;

	struct {
	       vehicle_type_T   car_type;
	       int              car_mpg;
	       char            *car_plate;
	} car;

} vehicle_T;

sm_conf_definition_T const
vehicle_type_names[] =
{
{ SM_CONF_DEF_MAGIC,
	"airplane",		sm_conf_type_choice_value,
	AIRPLANE,
	0, NULL, 0, NULL, NULL, NULL, NULL },
{ SM_CONF_DEF_MAGIC,
	"boat",		sm_conf_type_choice_value,
	BOAT,
	0, NULL, 0, NULL, NULL, NULL, NULL },
{ SM_CONF_DEF_MAGIC,
	"car",		sm_conf_type_choice_value,
	CAR,
	0, NULL, 0, NULL, NULL, NULL, NULL },

CONF_SENTINEL
};

static sm_conf_definition_T const
air_definitions[] =
{
	{ SM_CONF_DEF_MAGIC, "flight", sm_conf_type_string,
		offsetof(vehicle_T, airplane.air_flight), 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "passengers", sm_conf_type_u32,
		offsetof(vehicle_T, airplane.air_passengers),
		sizeof(unsigned int), "1"
	},

CONF_SENTINEL
};

static sm_conf_definition_T const
boat_definitions[] =
{
	{ SM_CONF_DEF_MAGIC, "name", sm_conf_type_string,
		offsetof(vehicle_T, boat.boat_name), 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "knots", sm_conf_type_u32,
		offsetof(vehicle_T, boat.boat_knots),
		sizeof(unsigned int),  NULL
	},

CONF_SENTINEL
};

static sm_conf_definition_T const
car_definitions[] =
{
	{ SM_CONF_DEF_MAGIC, "plate", sm_conf_type_string,
		offsetof(vehicle_T, car.car_plate), 0, NULL
	},

	{ SM_CONF_DEF_MAGIC, "mpg", sm_conf_type_u32,
		offsetof(vehicle_T, car.car_mpg),
		sizeof(unsigned int),  NULL
	},

CONF_SENTINEL
};

#if 0
/*
**  Vehicles are distinguished by section label.
*/

static sm_conf_definition_T
vehicle_definitions[] =
{

	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_type,
		offsetof(vehicle_T, vehicle_type),
		sizeof(vehicle_type_T)
	},

	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_choice,
		AIRPLANE,
		sizeof(vehicle_T),  "airplane", 0,
		air_definitions
	},

	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_choice,
		BOAT,
		sizeof(vehicle_T), "boat", 0,
		boat_definitions
	},

	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_choice,
		CAR,
		sizeof(vehicle_T),  "car", 0,
		car_definitions
	},

CONF_SENTINEL
};
#endif

typedef struct
{
	int	 sspec_type;
	short	 sspec_port;
	ipv4_T	 sspec_addr;
	char	*sspec_host;
} smtps_T;

typedef struct
{
	char const	*s_socket;
	smtps_T		 s_smtps;
	vehicle_T	 s_garage;
} mts_T;

static sm_conf_definition_T mta_def[] =
{
	{ SM_CONF_DEF_MAGIC, "port", sm_conf_type_u32,
		offsetof(smtps_T, sspec_port),
		sizeof(short),
		"1234", SM_CONF_FLAG_KEEP_DEFAULT
	},

	{ SM_CONF_DEF_MAGIC, "address", sm_conf_type_ipv4,
		offsetof(smtps_T, sspec_addr),
		sizeof(ipv4_T),
		"127.0.0.1", SM_CONF_FLAG_KEEP_DEFAULT
	},

	{ SM_CONF_DEF_MAGIC, "host", sm_conf_type_string,
		offsetof(smtps_T, sspec_host),
		0,
		NULL, 0
	},

CONF_SENTINEL
};

sm_conf_definition_T
mts_def[] =
{
	{ SM_CONF_DEF_MAGIC, "socket", sm_conf_type_string,
		offsetof(mts_T, s_socket), 0, "path/to/socket"
	},

	{ SM_CONF_DEF_MAGIC, "smtps", sm_conf_type_section,
		offsetof(mts_T, s_smtps), sizeof(mts_T),
		NULL,
		SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT,
		mta_def
	},

/*
	{ SM_CONF_DEF_MAGIC, "vehicle", sm_conf_type_union,
		offsetof(mts_T, s_garage), sizeof(vehicle_T),
		NULL, 0,
		vehicle_definitions,
		NULL,  NULL,
		"A red vehicle."
	},
*/

CONF_SENTINEL
};

sm_conf_definition_T
global_def[] =
{
	{ SM_CONF_DEF_MAGIC, "socket", sm_conf_type_string,
		offsetof(mts_T, s_socket), 0, "path/to/socket"
	},

CONF_SENTINEL
};

static mts_T mts;

static void
print_ipv4(ipv4_T ipv4, const char *name)
{
	sm_ret_T ret;
	sm_str_P ipv4s;

	ipv4s = sm_str_new(NULL, 20, 32);
	if (ipv4s == NULL) {
		printf("out of memory\n");
		return;
	}
	ret = sm_inet_ipv4str(ipv4, ipv4s);
	if (ret != SM_SUCCESS) {
		printf("sm_inet_ipv4str=0x%x\n", ret);
		return;
	}
	printf("%s= %s\n", name, sm_str_getdata(ipv4s));
}

static void
print_smtps(smtps_T *ss, const char *name, const char *s)
{
	printf("\nsmtps= %s\n", name);
	printf("name= %s\n", s == NULL ? "(none)" : s);
	printf("type= %u\n", ss->sspec_type);
	printf("port= %hu\n", ss->sspec_port);
	printf("host= \"%s\"\n", (ss->sspec_host == NULL) ? "(NULL)"
				: ss->sspec_host);
	print_ipv4(ss->sspec_addr, "address");

	printf("\n");
}

static void
print_mts(mts_T *st)
{
	printf("socket= \"%s\"\n",	st->s_socket);
	switch (st->s_garage.vehicle_type) {
	  case AIRPLANE:
		printf("airplane=\"%s\"\n",
			st->s_garage.airplane.air_flight);
		break;
	  case BOAT:
		printf("boat=\"%s\"\n",
			st->s_garage.boat.boat_name);
		break;
	  case CAR:
		printf("car=\"%s\"\n",
			st->s_garage.car.car_plate);
		break;
	  default:
		break;
	}
	print_smtps(&(st->s_smtps), "MTS", NULL);
}

static int
process(char const *name, FILE *fp, const char *ssname)
{
	sm_conf_T *stream;
	int err;
	char const *service_name;
	size_t service_name_n;
	sm_conf_iterator_T service_iter;
	smtps_T smtps;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		fprintf(stderr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		fprintf(stderr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	if ((err = sm_conf_scan(stream, mts_def, 0, &mts)) != 0) {
		fprintf(stderr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}
	print_mts(&mts);

	memset(&smtps, 0, sizeof(smtps));
	if (ssname != NULL)
		(void) snprintf(buf, sizeof(buf), "smtps{%s}", ssname);
	else
		(void) snprintf(buf, sizeof(buf), "smtps");

	service_iter = NULL;
	while ((err = sm_conf_scan_next(stream, buf, mta_def, 0,
				&service_name, &service_name_n,
				&smtps, &service_iter)) == 0)
	{
		print_smtps(&smtps, ssname, service_name);
	}

#if 0
	sm_conf_destroy(stream);
#endif /* 0 */

	return 0;
}

static int
gprocess(char const *name)
{
	sm_conf_T *stream;
	int err;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		fprintf(stderr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, NULL)) != 0) {
		fprintf(stderr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	err = sm_conf_scan(stream, global_def, SM_CONF_FLAG_ALLOW_ANY_SECTION, &mts);
	if (err != 0) {
		fprintf(stderr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}

	return 0;
}

int
main(int ac, char **av)
{
	int ai, c, ret, done;
	char *ssname;

	done = 0;
	ssname = NULL;
	while ((c = getopt(ac, av, "f:g:N:o:v:")) != -1) {
		switch (c) {
		  case 'f':
			ret = process(optarg, NULL, ssname);
			if (ret != 0)
				return ret;
			done = 1;
			break;
		  case 'g':
			ret = gprocess(optarg);
			if (ret != 0)
				return ret;
			return 0;
			break;
		  case 'N':
			ssname = strdup(optarg);
			if (ssname == NULL)
				return ENOMEM;
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (done) {
		print_mts(&mts);
		return 0;
	}

	if (ac == 0)
		return process("*stdin*", stdin, ssname);

	for (ai = 0; ai < ac; ai++) {
		int ret = process(av[ai], NULL, ssname);
		if (ret != 0)
			return ret;
	}
	return 0;
}
