/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-ips-0.c,v 1.2 2012/11/17 21:18:38 ca Exp $")

#include "sm/string.h"
#include "sm/net.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include "sm/sockcnf.h"
#define SM_SOCKCNFDEF 1
#include "sm/sockcnfdef.h"
#include "sm/io.h"

#define MT_IPS_MAX	8

struct mt_ip_S {
	sm_ipaddr_T	 ipaddr;
	uint32_t	 ip_pref;
};
typedef struct mt_ip_S mt_ip_T, *mt_ip_P;


typedef struct mt_cnf_S		mt_cnf_T, *mt_cnf_P;
struct mt_cnf_S
{
	sm_magic_T	 sm_magic;
	mt_ip_T		 mtips_ip[MT_IPS_MAX];
	uint		 mtips_n;
};

sm_conf_definition_T mt_ip_def[] = {
{ SM_CONF_DEF_MAGIC, "ip", sm_conf_type_ip,
	offsetof(mt_ip_T, ipaddr), sizeof(sm_ipaddr_T),
	NULL, 0,
	NULL, NULL,	NULL,
	"IP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "pref",		sm_conf_type_u32,
	offsetof(mt_ip_T, ip_pref),	sizeof(uint32_t),
	NULL, 0,
	NULL, NULL,	NULL,
	"pref"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

	CONF_SENTINEL
};

sm_conf_definition_T mt_ip_definitions[] = {
{ SM_CONF_DEF_MAGIC, "",	sm_conf_type_section,
	offsetof(mt_cnf_T, mtips_ip), sizeof(mt_ip_T),
	NULL, SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT,
	mt_ip_def,
	NULL, NULL, "ips" },

{ SM_CONF_DEF_MAGIC, "n",	sm_conf_type_array_n,
	offsetof(mt_cnf_T, mtips_n), sizeof(uint),
	NULL, 0,
	mt_ip_def,
	NULL, NULL,
	"number of ips (implicit)" },

CONF_SENTINEL
};

sm_conf_definition_T mt_defs[] = {

{ SM_CONF_DEF_MAGIC, "dest",		sm_conf_type_array,
	0,	/* offset */
	MT_IPS_MAX,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_FLAT,
	mt_ip_definitions,
	NULL,  NULL,
	"ip declarations"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};


static mt_cnf_T mt_ips;

static void
print_ips(mt_cnf_T *st)
{
	uint u, n;

	n = st->mtips_n;
	for (u = 0; u < n; u++) {
		sm_io_fprintf(smioout, "dest { ip=%I; pref=%d; }\n",
			&(st->mtips_ip[u].ipaddr),
			st->mtips_ip[u].ip_pref
		);
	}
	sm_io_flush(smioout);
}

static int
process(char const *name, FILE *fp, int show)
{
	sm_conf_T *stream;
	int err;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		sm_io_fprintf(smioerr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		sm_io_fprintf(smioerr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	if ((err = sm_conf_scan(stream, mt_defs, 0, &mt_ips)) != 0) {
		sm_io_fprintf(smioerr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}
	print_ips(&mt_ips);
	if (show) {
		err = sm_conf_prt_conf(mt_defs, &mt_ips, smioout);
		sm_io_flush(smioout);
	}

#if 0
	sm_conf_destroy(stream);
#endif /* 0 */

	return 0;
}

static void
showversion(void)
{
	sm_io_fprintf(smioerr, "compile time options:\n"
#if MTA_NETINET6
	"MTA_NETINET6\n"
#endif
	);
}

int
main(int ac, char **av)
{
	int ai, c, ret, done, show;

	done = 0;
	show = 0;
	while ((c = getopt(ac, av, "f:sV")) != -1) {
		switch (c) {
		  case 'f':
			ret = process(optarg, NULL, show);
			if (ret != 0)
				return ret;
			done = 1;
			break;
		  case 's':
			show = 1;
			break;
		  case 'V':
			showversion();
			exit(0);
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (done) {
		print_ips(&mt_ips);
		return 0;
	}

	if (ac == 0)
		return process("*stdin*", stdin, show);

	for (ai = 0; ai < ac; ai++) {
		ret = process(av[ai], NULL, show);
		if (ret != 0)
			return ret;
	}
	return 0;
}
