/*
 * Copyright (c) 2010 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: gensig.c,v 1.9 2011/01/30 21:49:13 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/dkim.h"
#include "dkim.h"

#if MTA_USE_DKIM

/*
**  DKIM_GENSIG -- generate a DKIM signatute
**
**	Parameters:
**		dkim_ctx -- DKIM context
**		dkim_msg -- DKIM msg context
**		dkim_cookie -- DKIM cookie (header "file")
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
dkim_gensig(dkim_ctx_P dkim_ctx, dkim_msg_P dkim_msg, dkim_cookie_P dkim_cookie)
{
	sm_ret_T ret;
	int nid, status;
	uint l, digestlen;
	uchar *digest;
	dkim_s_P dkim_s;

	SM_REQUIRE(dkim_ctx != NULL);
	dkim_s = dkim_msg->dkm_s;
	SM_ASSERT(dkim_s != NULL);

	dkim_msg->dkm_rsaout = sm_zalloc(dkim_s->dks_rsa_keysize);
	if (NULL == dkim_msg->dkm_rsaout) {
		ret = sm_error_perm(SM_EM_DKIM, ENOMEM);
		goto error;
	}

	if (dkim_cookie->dkf_sha256) {
		nid = NID_sha256;
		digest = DKIM_SHA256_OUT(dkim_cookie);
		digestlen = sizeof(DKIM_SHA256_OUT(dkim_cookie));
	}
	else {
		nid = NID_sha1;
		digest = DKIM_SHA1_OUT(dkim_cookie);
		digestlen = sizeof(DKIM_SHA1_OUT(dkim_cookie));
	}
	PRTBUF("digest", digest, digestlen);
	status = RSA_sign(nid, digest, digestlen, dkim_msg->dkm_rsaout, &l, dkim_s->dks_rsa);
	if (status == 0 || l == 0) {
		ret = sm_error_perm(SM_EM_DKIM, EINVAL);
		goto error;
	}

	dkim_msg->dkm_rsaoutlen = l;

	return SM_SUCCESS;

  error:
	if (dkim_msg->dkm_rsaout != NULL) {
		sm_free(dkim_msg->dkm_rsaout);
		dkim_msg->dkm_rsaout = NULL;
	}
	return ret;
}
#endif /* MTA_USE_DKIM */
