#!/bin/sh
# $Id: t-dns-1.sh,v 1.20 2014/02/26 16:56:09 ca Exp $
# Copyright (c) 2004-2006 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# run t-dnstsk-1 with various parameters/arguments, esp. CNAME handling.
# depends on meta1.org DNS data.
# ----------------------------------------
#

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0 {MTA_NO_DNS_TEST} is set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../
fi
. ${SD}/common.sh

OPTS="-Vam"
TPRG=./t-dnstsk-1

if test X"${MTA_DNS_TIMEOUT}" != "X"
then
  OPTS="${OPTS} -O ${MTA_DNS_TIMEOUT}"
fi

TEST=0

# ----------------------------------------
# test very long hostname
TEST=`expr $TEST + 1 `
cat > ${OK} <<EOF
IPv4=7F7F7F01
EOF
NAME="mxlong.meta1.org"
if ${TPRG} -V "$@" -i ${NS} ${NAME} | egrep 'IPv4|DNSR|CNAME' > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

if test X"${MTA_TEST_DNS_TIMING}" != "X"
then

cat > ${OK} <<EOF
IPv4=7F000001
EOF
# ----------------------------------------
for NAME in cname1.meta1.org cname2cname1.meta1.org cname3cname2.meta1.org cname4cname3.meta1.org
do
TEST=`expr $TEST + 1 `
if ${TPRG} ${OPTS} "$@" -i ${NS} ${NAME} | grep IPv4 > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test ${TEST} failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
done

# 2004-05-17
# seems newer bind resolvers don't like this...
# cat > ${OK} <<EOF
# DNSR_TEMP
# EOF
# if ${TPRG} ${OPTS} "$@" -i ${NS} cname2itself.meta1.org | egrep 'IPv4|DNSR|CNAME' > ${OUT}
# then
#   if diff ${OK} ${OUT}
#   then
#     :
#   else
#     ERRS=`expr ${ERRS} + 1 `
#     test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
#   fi
# else
#   ERRS=`expr ${ERRS} + 1 `
#   test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
# fi

# ----------------------------------------
TEST=`expr $TEST + 1 `
cat > ${OK} <<EOF
DNSR_TEMP
EOF
NAME=cnameloop1.meta1.org
if ${TPRG} ${OPTS} "$@" -i ${NS} ${NAME} | egrep 'IPv4|DNSR|CNAME' > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

# ----------------------------------------
TEST=`expr $TEST + 1 `
cat > ${OK} <<EOF
DNSR_NOTFOUND
DNSR_NOTFOUND
EOF
NAME="mxip.meta1.org mxip2.meta1.org"
if ${TPRG} ${OPTS} "$@" -i ${NS} ${NAME} | egrep 'IPv4|DNSR|CNAME' > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

# ----------------------------------------
TEST=`expr $TEST + 1 `
cat > ${OK} <<EOF
DNSR_TEMP
EOF
NAME="mx.bogus.meta1.org"
if ${TPRG} ${OPTS} "$@" -i ${NS} ${NAME} | egrep 'IPv4|DNSR|CNAME' > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

# MTA_TEST_DNS_TIMING
fi

exit ${ERRS}
