/*
 */

#include "sm/generic.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/binsearch.h"

/*
sort table by length of ids and only search in those parts
initialize some length arry with low/high values to use those as
starting points? for example:
struct length_S { int l_low, l_high; }
length_S lengths[0..max_len - 1];
init lengths based on idtab...
low = lengths[id_len].l_low;
high = lengths[id_len].l_high;

this is only worth the effort is the table is reasonably large
and used many time.
otherwise the overhead for initialization might be even bigger.
however, here we can use strncmp,
i.e., strings don't need to be NUL terminated.
 */

sm_ret_T
sm_binleninit(sm_identry_P idtab, uint sz, sm_binlen_P bltab, uint blsz)
{
	int i, llen, clen;

	SM_REQUIRE(idtab != NULL);
	SM_REQUIRE(sz > 0);
	SM_REQUIRE(bltab != NULL);
	SM_REQUIRE(blsz > 0);

#if 0
/* require static initialization? */
	for (i = 0; i < blsz; i++) {
		bltab[i].bl_first = 1;
		bltab[i].bl_last = 0;
	}
#endif

	llen = strlen(idtab[0].idt_id);
	if (llen > blsz)
		return SM_SUCCESS;	/* some error instead? */

	bltab[llen].bl_first = 0;
	for (i = 0; i < sz; i++) {
		clen = strlen(idtab[i].idt_id);
		if (clen == llen)
			continue;

		bltab[llen].bl_last = i - 1;
		if (clen > blsz)
			return SM_SUCCESS;
		bltab[clen].bl_first = i;
		llen = clen;
	}
	bltab[llen].bl_last = i - 1;
	return SM_SUCCESS;
}

sm_ret_T
sm_binlensrch(sm_identry_P idtab, uint sz, sm_binlen_P bltab, uint blsz, const char *id, uint idlen, int *pidx)
{
	int cmp, low, mid, high;

	SM_REQUIRE(id != NULL);
	SM_REQUIRE(pidx != NULL);
	SM_REQUIRE(idtab != NULL);
	SM_REQUIRE(sz > 0);

	if (idlen >= blsz) {
		low = 0;
		high = sz - 1;
	}
	else {
		low = bltab[idlen].bl_first;
		high = bltab[idlen].bl_last;
	}
	while (low <= high) {
		mid = (low + high) / 2;
		cmp = strcmp(id, idtab[mid].idt_id);
		if (cmp == 0) {
			*pidx = mid;
			return SM_SUCCESS;
		}
		else if (cmp < 0)
			high = mid - 1;
		else if (cmp > 0)
			low = mid + 1;
	}
	return sm_err_perm(SM_E_NOTFOUND);
}
