/*
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: printit.c,v 1.2 2013/12/07 03:41:13 ca Exp $
 */


	/*
	 *  All reasonable formats wind up here.  At this point, `cp'
	 *  points to a string which (if not flags&LADJUST) should be
	 *  padded out to `width' places.  If flags&ZEROPAD, it should
	 *  first be prefixed by any sign or other prefix; otherwise,
	 *  it should be blank padded before the prefix is emitted.
	 *  After any left-hand padding and prefixing, emit zeroes
	 *  required by a decimal [diouxX] precision, then print the
	 *  string proper, then emit zeroes required by any leftover
	 *  floating precision; finally, if LADJUST, pad with blanks.
	 *
	 *  Compute actual size, so we know how much to pad.
	 *  size excludes decimal prec; realsz includes it.
	 */

	realsz = dprec > size ? dprec : size;
	if (sign)
		realsz++;
	else if (flags & HEXPREFIX)
		realsz+= 2;

	/* right-adjusting blank padding */
	if ((flags & (LADJUST|ZEROPAD)) == 0)
		PAD(width - realsz, blanks);

	/* prefix */
	if (sign)
		VFP_PRINT(&sign, 1);
	else if (flags & HEXPREFIX) {
		ox[0] = '0';
		ox[1] = ch;
		VFP_PRINT(ox, 2);
	}

	/* right-adjusting zero padding */
	if ((flags & (LADJUST|ZEROPAD)) == ZEROPAD)
		PAD(width - realsz, zeroes);

	/* leading zeroes from decimal precision */
	PAD(dprec - size, zeroes);

	if ((flags & SANITIZE) != 0 && (osize != size || prec >= 0)) {
		int limit;	/* limit for fixed size files */
		char pfix;

		if (f_flags(*fp) & (SMSTR|SMSTRSTR))
			limit = SM_MIN(f_w(*fp), size);
		else
			limit = size;
		pfix = encode[0] != '\0' ? encode[0] : '%';

		VFP_FLUSH();
		xdigs = "0123456789ABCDEF";
		for (m = 0, n = 0; m < osize && n < limit; m++, n++) {
			n2 = cp[m];
			if (OKTOPRINT(n2, encode)) {
				if (sm_is_err(sm_io_putc(fp, n2)))
					goto error;
			}
			else {
				if (n + 3 > limit) {
					while (n < limit) {
						if (sm_is_err(sm_io_putc(fp, (int) '_')))
							goto error;
						++n;
					}
					break;
				}
				if (sm_is_err(sm_io_putc(fp, (int) pfix)))
					goto error;
				if (sm_is_err(sm_io_putc(fp, xdigs[(n2 >> 4) & 15])))
					goto error;
				if (sm_is_err(sm_io_putc(fp, xdigs[n2 & 15])))
					goto error;
				n += 2;
			}
		}
	}
	else {
		/* the string or number proper */
		VFP_PRINT(cp, size);
	}
	/* left-adjusting padding (always blank) */
	if (flags & LADJUST)
		PAD(width - realsz, blanks);

	/* finally, adjust ret */
	ret += width > realsz ? width : realsz;
  flushit:
	VFP_FLUSH();	/* copy out the I/O vectors */

