/*
 * Copyright (c) 2004-2006 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: smerr2txt.c,v 1.40 2014/02/15 03:23:02 ca Exp $")

#include "sm/error.h"
#include "sm/wait.h"
#include "sm/dns.h"
#include "sm/bdb.h"
#include "sm/rfc2821.h"
#define SMERR2TXT 1
#include "smerr2txt.h"

#if 0
/*
convert define line to struct entry:
:'a,.s/^#define \(SM[^ 	]*\).*/	{ \1,	"\1",	-1	},/
*/
#endif /* 0 */

static e2s_T
e_modules[] =
{
	{ SM_EM_NONE,		"none",	-1	},
	{ SM_EM_AQ,		"AQ",	-1	},
	{ SM_EM_IQDB,		"IQDB",	-1	},
	{ SM_EM_IBDB,		"IBDB",	-1	},
	{ SM_EM_EDB,		"EDB",	-1	},
	{ SM_EM_CDB,		"CDB",	-1	},
	{ SM_EM_DA,		"delivery agent",	-1	},
	{ SM_EM_OCC,		"open connection cache",	-1	},
	{ SM_EM_DNS,		"DNS",	-1	},
	{ SM_EM_SMTPC,		"SMTPC",	-1	},
	{ SM_EM_SMTPS,		"SMTPS",	-1	},
	{ SM_EM_MAPS,		"MAPS",	-1	},
	{ SM_EM_MAP,		"MAP",	-1	},
	{ SM_EM_BDB,		"BDB",	-1	},
	{ SM_EM_STTHRIO,	"STTHRIO",	-1	},
	{ SM_EM_STTHR,		"STTHR",	-1	},
	{ SM_EM_EVTHR,		"EVTHR",	-1	},
	{ SM_EM_RECCOM,		"RECCOM",	-1	},
	{ SM_EM_HT,		"hash table",	-1	},
	{ SM_EM_RSC,		"restricted size cache",	-1	},
	{ SM_EM_STR,		"str",	-1	},
	{ SM_EM_RCB,		"RCB",	-1	},
	{ SM_EM_TLS,		"TLS",	-1	},
	{ SM_EM_SASL,		"SASL",	-1	},
	{ SM_EM_NET,		"NET",	-1	},
	{ SM_EM_IO,		"IO",	-1	},
	{ SM_EM_LOG,		"logging",	-1	},
	{ SM_EM_ADDR,		"RFC 2821 address parsing",	-1	},
	{ SM_EM_IP,		"IP address parsing",	-1	},
	{ SM_EM_Q_Q2AR,		"QMGR: QMGR to SMAR",	-1	},
	{ SM_EM_Q_AR2Q,		"QMGR: SMAR to QMGR",	-1	},
	{ SM_EM_Q_Q2SS,		"QMGR: QMGR to SMTPS",	-1	},
	{ SM_EM_Q_SS2Q,		"QMGR: SMTPS to QMGR",	-1	},
	{ SM_EM_Q_Q2SC,		"QMGR: QMGR to SMTPC",	-1	},
	{ SM_EM_Q_SC2Q,		"QMGR: SMTPC to QMGR",	-1	},
	{ SM_EM_Q_RDIBDB,	"QMGR: read IBDB",	-1	},
	{ SM_EM_Q_IQDB,		"QMGR: IQDB",	-1	},
	{ SM_EM_Q_IBDB,		"QMGR: IBDB",	-1	},
	{ SM_EM_Q_EDBC,		"QMGR: EDBC",	-1	},
	{ SM_EM_Q_DSN,		"QMGR: DSN",	-1	},
	{ SM_EM_Q_START,	"QMGR: startup",	-1	},
	{ SM_EM_Q_CTL2Q,	"QMGR: control",	-1	},
	{ SM_EM_Q_CONF,		"QMGR: configuration",	-1	},
	{ SM_EM_Q_CLEAN,	"QMGR: cleanup",	-1	},
	{ SM_EM_Q_SCHED,	"QMGR: scheduler",	-1	},
	{ SM_EM_Q,		"QMGR: misc",	-1	},
	{ SM_EM_AR_Q2AR,	"SMAR: QMGR to SMAR",	-1	},
	{ SM_EM_AR_AR2Q,	"SMAR: SMAR to QMGR",	-1	},
	{ SM_EM_AR_DNSW,	"SMAR: DNS write",	-1	},
	{ SM_EM_AR_DNSR,	"SMAR: DNS read",	-1	},
	{ SM_EM_AR_DNSD,	"SMAR: DNS read",	-1	},
	{ SM_EM_AR_WAIT,	"SMAR: wait for async function",	-1	},
	{ SM_EM_AR,		"SMAR: misc",	-1	},
	{ SM_EM_CONF,		"configuration",	-1	},
	{ SM_EM_PMILTER,	"pmilter",	-1	},
	{ SM_EM_THR,		"thread",	-1	},
	{ SM_EM_BATV,		"BATV",	-1	},
	{ SM_EM_DKIM,		"DKIM",	-1	},
	{ SM_EM_UTIL,		"utility functions",	-1	},
	{ 0,			NULL	,	-1	}
};

e2s_T
e_error[] =
{
	{ SM_E_NOMORE,		"no next entry",	-1	},
	{ SM_E_UNEXPECTED,	"unexpected",	-1	},
	{ SM_E_RANGE,		"input parameter out of range",	-1	},
	{ SM_E_2BIG,		"input parameter too big",	-1	},
	{ SM_E_FULL,		"object full",	-1	},
	{ SM_E_OVFLW_SC,	"scalar overflow",	-1	},
	{ SM_E_OVFLW_NS,	"nonscalar overflow",	-1	},
	{ SM_E_NOTFOUND,	"not found",	-1	},
	{ SM_E_STARTFAIL,	"start failure",	-1	},
	{ SM_E_SYNTAX,		"syntax",	-1	},
	{ SM_E_USAGE,		"usage",	-1	},
	{ SM_E_DONTSTART,	"donotstart",	0	},
	{ SM_E_VER_MIX,		"version mismatch",	-1	},
	{ SM_E_NO_AR,		"smar not available",	-1	},
	{ SM_E_NO_DA,		"no delivery agent available",	-1	},
	{ SM_E_NO_DA_FREE,	"no free delivery agent",	-1	},
	{ SM_E_PR_V_MISM,	"protocol version mismatch",	-1	},
	{ SM_E_PR_ERR,		"protocol error",	-1	},
	{ SM_E_RCB2LONG,	"RCB too large",	-1	},
	{ SM_E_TEMPMAP,		"temporary map error",	-1	},
	{ SM_E_PERMMAP,		"permanent map error",	-1	},
	{ SM_E_CLOSEDMAP,	"closedmap",	-1	},
	{ SM_E_NOMAP,		"map is not available",	-1	},
	{ SM_E_NOTIMPL,		"not implemented",	-1	},
	{ SM_E_MXEMPTY,		"MX list is empty after removal of local addresses",	-1	},
	{ SM_E_TTMYSELF,	"talking to myself",	-1	},
	{ SM_E_ALIASEXP,	"alias expansion failed",	-1	},
	{ SM_E_ALIAS_REC,	"aliases nested too deep",	-1	},
	{ SM_E_IP_MISM,	"DNS answer from different server",	-1	},
	{ SM_E_NOSIG,	"no signature",	-1	},
	{ SM_E_EXPIRED,	"expired",	-1	},
	{ SM_E_RSR_PRB,	"resource problem",	-1	},
	{ SM_E_ILL_PIPE,	"illegal pipelining",	-1	},
	{ SM_E_RD,	"read error",	-1	},
	{ SM_E_EOF,	"EOF",	-1	},
	{ SM_E_WR,	"write error",	-1	},
	{ SM_E_BARE_NL,	"bare NL detected",	-1	},
	{ SM_E_CONN_CLSD,	"internal connection to other module closed", -1 },
	{ SM_E_WRONG_ID,	"internal error: communication context id mismatch",	-1	},
	{ SM_SSL_ERROR_SYSCALL,	"TLS syscall error", -1 },
	{ SM_SSL_ERROR_SSL,	"TLS protocol error", -1 },
	{ SM_SSL_ERROR_GENERIC,	"TLS unknown error", -1 },
	{ SM_E_UNAVAIL,	"unavailable", -1 },
	{ SM_E_KEYEXIST,	"key exists", -1 },
	{ SM_E_FROMPARSE,	"From: parsing error", -1 },
	{ SM_E_TEST,	"just a test error", -1 },

	{ DNSR_NOTFOUND,	"DNS record not found (authorative)", -1 },
	{ DNSR_NO_DATA,	"valid name but no data record of requested type", -1 },
	{ DNSR_TEMP,	"query failed: try again", -1 },
	{ DNSR_PERM,	"query failed: do not retry", -1 },
	{ DNSR_TIMEOUT,	"query timed out", -1 },
	{ DNSR_REFUSED,	"query refused by nameserver", -1 },
	{ DNSR_MXINVALID,	"MX record invalid",	-1	},
	{ DNSR_PTRINVALID,	"PTR record invalid",	-1	},
	{ DNSR_CNINVALID,	"CNAME record invalid",	-1	},

	{ R2821_ERR_LEFT,	"missing '<'", -1 },
	{ R2821_ERR_RIGHT,	"missing '>'", -1 },
	{ R2821_ERR_ROUTE,	"route address not allowed", -1 },
	{ R2821_ERR_HOST,	"domain part does not start with an atom", -1 },
	{ R2821_ERR_FQDN,	"domain part is not a FQDN", -1 },
	{ R2821_ERR_LOCAL,	"missing local part", -1 },
	{ R2821_ERR_AT,	"missing '@'", -1 },
	{ R2821_ERR_EMPTY,	"<> not allowed here", -1 },
	{ R2821_ERR_MBOX,	"completely empty address", -1 },
	{ R2821_ERR_COLON,	"missing ':' (unused)", -1 },
	{ R2821_ERR_DOTSTR,	"\"dotstr\" syntax is wrong: expected atom after '.'", -1 },
	{ R2821_ERR_DOMAIN,	"domain part is empty", -1 },
	{ R2821_ERR_DOT,	"expecting ':'", -1 },
	{ R2821_ERR_TRAILDOT,	"domain has trailing '.'", -1 },
	{ R2821_ERR_ATDOMAIN,	"'@' in route address missing", -1 },
	{ R2821_ERR_COMMA,	"comma in route address missing", -1 },
	{ R2821_ERR_DOMSYNTAX,	"domain part has wrong syntax", -1 },
	{ R2821_ERR_MORE,	"tokens after end", -1 },
	{ R2821_ERR_DEL_MISS,	"domain does not end in delimiter", -1 },
	{ R2821_ERR_TOOLONG,	"address too long", -1 },
	{ R2821_ERR_TRAILBACKSL,	"trailing backslash", -1 },
	{ R2821_ERR_CLS_BRACKET,	"too many closing brackets", -1 },
	{ R2821_ERR_CLS_PARENTH,	"too many closing parentheses", -1 },
	{ R2821_ERR_PARENTHESIS,	"parenthesis in address not allowed", -1 },
	{ R2821_ERR_MUSTQUOTE,	"character must be quoted but isn't", -1 },

	{ SM_CNF_E_NEWLINE_IN_STRING,	"newline in string", -1 },
	{ SM_CNF_E_EOF_IN_STRING,	"EOF in string", -1 },
	{ SM_CNF_E_BAD_CHAR,	"unexpected character", -1 },
	{ SM_CNF_E_NO_MEMORY,	"out of memory", -1 },
	{ SM_CNF_E_HEX_EXPECTED,	"hexadecimal digit expected after \\x \\u or \\U", -1 },
	{ SM_CNF_E_CHAR_OVERFLOW,	"character constant too large", -1 },
	{ SM_CNF_E_INVALID,	"invalid argument to configuration function", -1 },
	{ SM_CNF_E_READ,	"error while reading configuration file", -1 },
	{ SM_CNF_E_READ_OPEN,	"cannot open configuration file for reading", -1 },
	{ SM_CNF_E_READ_CLOSE,	"error closing configuration file after reading", -1 },
	{ SM_CNF_E_SYNTAX,	"syntax error", -1 },
	{ SM_CNF_E_NOT_FOUND,	"not found", -1 },
	{ SM_CNF_E_ALREADY,	"already exists", -1 },
	{ SM_CNF_E_TYPE,	"invalid value", -1 },
	{ SM_CNF_E_NUL_IN_STRING,	"ascii NUL character in string", -1 },
	{ SM_CNF_E_TOO_MANY,	"too many elements in array", -1 },

	{ 0,			NULL	,	-1	}
};

/* XXX HACK XXX is there a DB function to do this? */

e2s_T
e_bdb4_2[] =
{
	{ DB_DONOTINDEX,	"DB_DONOTINDEX", -1 },
#ifdef DB_FILEOPEN
	{ DB_FILEOPEN,	"DB_FILEOPEN", -1 },
#endif
	{ DB_KEYEMPTY,	"DB_KEYEMPTY", -1 },
	{ DB_KEYEXIST,	"DB_KEYEXIST", -1 },
	{ DB_LOCK_DEADLOCK,	"DB_LOCK_DEADLOCK", -1 },
	{ DB_LOCK_NOTGRANTED,	"DB_LOCK_NOTGRANTED", -1 },
	{ DB_NOSERVER,	"DB_NOSERVER", -1 },
	{ DB_NOSERVER_HOME,	"DB_NOSERVER_HOME", -1 },
	{ DB_NOSERVER_ID,	"DB_NOSERVER_ID", -1 },
	{ DB_NOTFOUND,	"DB_NOTFOUND", -1 },
	{ DB_OLD_VERSION,	"DB_OLD_VERSION", -1 },
	{ DB_PAGE_NOTFOUND,	"DB_PAGE_NOTFOUND", -1 },
	{ DB_REP_DUPMASTER,	"DB_REP_DUPMASTER", -1 },
#ifdef DB_REP_HANDLE_DEAD
	{ DB_REP_HANDLE_DEAD,	"DB_REP_HANDLE_DEAD", -1 },
#endif
	{ DB_REP_HOLDELECTION,	"DB_REP_HOLDELECTION", -1 },
#ifdef DB_REP_ISPERM
	{ DB_REP_ISPERM,	"DB_REP_ISPERM", -1 },
#endif
	{ DB_REP_NEWMASTER,	"DB_REP_NEWMASTER", -1 },
	{ DB_REP_NEWSITE,	"DB_REP_NEWSITE", -1 },
#ifdef DB_REP_NOTPERM
	{ DB_REP_NOTPERM,	"DB_REP_NOTPERM", -1 },
#endif
#ifdef DB_REP_OUTDATED
	{ DB_REP_OUTDATED,	"DB_REP_OUTDATED", -1 },
#endif
	{ DB_REP_UNAVAIL,	"DB_REP_UNAVAIL", -1 },
	{ DB_RUNRECOVERY,	"DB_RUNRECOVERY", -1 },
	{ DB_SECONDARY_BAD,	"DB_SECONDARY_BAD", -1 },
#ifdef DB_VERIFY_BAD
	{ DB_VERIFY_BAD,	"DB_VERIFY_BAD", -1 },
#endif
#if defined(DB_BUFFER_SMALL) && ENOMEM != DB_BUFFER_SMALL
	{ DB_BUFFER_SMALL,	"DB_BUFFER_SMALL", -1 },
#endif
	{ 0,			NULL	,	-1	}
};

#if 0
//static e2s_T
//e_dns[] =
//{
//	{ DNSR_NOTFOUND,	"DNSR Not Found" },
//	{ DNSR_NO_DATA,	"Valid name but no data record of requested type" },
//	{ DNSR_TEMP,	"query failed: try again" },
//	{ DNSR_PERM,	"query failed: don't retry" },
//	{ DNSR_TIMEOUT,	"query timed out" },
//	{ DNSR_REFUSED,	"query refused by nameserver" },
//	{ DNSR_MXINVALID,	"MX record invalid"	},
//	{ DNSR_PTRINVALID,	"PTR record invalid"	},
//	{ DNSR_CNINVALID,	"CNAME record invalid"	},
//	{ 0,			NULL		}
//};
#endif /* 0 */

const char *
finde(int ecode, e2s_P table)
{
	int i;

	i = 0;
	while (table[i].e2s_code != 0 || table[i].e2s_str != NULL) {
		if (table[i].e2s_code == ecode)
			return table[i].e2s_str;
		if (sm_error_value(table[i].e2s_code) == sm_error_value(ecode))
			return table[i].e2s_str;
		++i;
	}
	return NULL;
}

const char *
smmod2txt(sm_error_T err)
{
	char *errtxt;

	errtxt = finde(sm_error_module(err), e_modules);
	return (errtxt != NULL) ? errtxt : SM_ETXT_UNKNOWN;
}

const char *
smerr2txt(sm_error_T err)
{
	int errornumber, eno;
	char *errtxt;

	eno = errornumber = sm_error_value(err);
	errtxt = NULL;

	if (err == SM_IO_EOF)
		errtxt = "EOF";
	else if (sm_is_ex_error(err)) {
		errtxt = ex2txt_r(sm_error_value(err));
	}
	else if (errornumber > 0 && errornumber < sm_sys_nerr)
		errtxt = strerror(errornumber);
	else if (SM_IS_BDB_ERR(err))
#if 0
//		/*
//		**  If this is used then every application needs to link
//		**  against libdb.
//		*/
//
//		errtxt = db_strerror(eno);
#else
		errtxt = finde(eno, e_bdb4_2);
#endif
#if 0
//	else if (errornumber >= DNS_ERR_BASE && errornumber < R2821_BASE)
//		errtxt = finde(err, e_dns);
#endif
	else
		errtxt = finde(errornumber, e_error);
	return (errtxt != NULL) ? errtxt : SM_ETXT_UNKNOWN;
}
