#!/bin/sh
# $Id: t-msgmod-0.sh,v 1.6 2006/10/05 04:27:40 ca Exp $
# Copyright (c) 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test message body modifications: appending/prepending headers
# uses t-msgmod-0
# ----------------------------------------
#
test -s stop && exit 1
#

GREP=grep
ERR=false
ERRS=0
if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

TPRG=./t-msgmod-0
IN=msg-0.in
OK=msg-0.ok
OUT=msg-0.out

cat > ${IN} <<EOF
From: a
To: b
Subject: c

body
.
EOF


# ----------------------------------------
# test: append header
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cat > ${OK} <<EOF
From: a
To: b
Subject: c
Append: headerA

body
.
EOF

for i in 1 2 3 4 5 6 7 8 9 10 16 32 128 256
do
  ${TPRG} -b ${i} -M 'a=Append: headerA' < ${IN} > ${OUT}
  if diff ${OK} ${OUT}
  then
     :
  else
     echo "$0: test ${TEST} failed for i=${i}"
     ERR=true
     break
  fi
done
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: prepend header
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cat > ${OK} <<EOF
Prepend: headerP
From: a
To: b
Subject: c

body
.
EOF

for i in 1 2 3 4 5 6 7 8 9 10 16 32 128 256
do
  ${TPRG} -b ${i} -M 'p=Prepend: headerP' < ${IN} > ${OUT}
  if diff ${OK} ${OUT}
  then
     :
  else
     echo "$0: test ${TEST} failed for i=${i}"
     ERR=true
     break
  fi
done
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: prepend and append header
TEST=3
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cat > ${OK} <<EOF
Prepend: headerP
From: a
To: b
Subject: c
Append: headerA

body
.
EOF

for i in 1 2 3 4 5 6 7 8 9 10 16 32 128 256
do
  ${TPRG} -b ${i} -M 'p=Prepend: headerP' -M 'a=Append: headerA' < ${IN} > ${OUT}
  if diff ${OK} ${OUT}
  then
     :
  else
     echo "$0: test ${TEST} failed for i=${i}"
     ERR=true
     break
  fi
done
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test: append headers
TEST=4
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cat > ${OK} <<EOF
From: a
To: b
Subject: c
Append: headerA
Append2: headerA2

body
.
EOF

for i in 1 2 3 4 5 6 7 8 9 10 16 32 128 256
do
  ${TPRG} -b ${i} -M 'a=Append: headerA'  -M 'a=Append2: headerA2' < ${IN} > ${OUT}
  if diff ${OK} ${OUT}
  then
     :
  else
     echo "$0: test ${TEST} failed for i=${i}"
     ERR=true
     break
  fi
done
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test: prepend headers
TEST=5
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cat > ${OK} <<EOF
Prepend: headerP
Prepend2: headerP2
From: a
To: b
Subject: c

body
.
EOF

for i in 1 2 3 4 5 6 7 8 9 10 16 32 128 256
do
  ${TPRG} -b ${i} -M 'p=Prepend: headerP'  -M 'p=Prepend2: headerP2' < ${IN} > ${OUT}
  if diff ${OK} ${OUT}
  then
     :
  else
     echo "$0: test ${TEST} failed for i=${i}"
     ERR=true
     break
  fi
done
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test: prepend and append headers
TEST=6
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cat > ${OK} <<EOF
Prepend: headerP
Prepend2: headerP2
From: a
To: b
Subject: c
Append: headerA
Append2: headerA2

body
.
EOF

for i in 1 2 3 4 5 6 7 8 9 10 16 32 128 256
do
  ${TPRG} -b ${i} -M 'p=Prepend: headerP' -M 'a=Append: headerA'  -M 'p=Prepend2: headerP2' -M 'a=Append2: headerA2' < ${IN} > ${OUT}
  if diff ${OK} ${OUT}
  then
     :
  else
     echo "$0: test ${TEST} failed for i=${i}"
     ERR=true
     break
  fi
done
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi



# ----------------------------------------
# end
if test "${ERRS}" = "0"
then
  rm -f ${IN} ${OK} ${OUT}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
